@ECHO OFF

:: Define Min Game Version
SET MIN_GAME_VERSION=31

:: Prevent Closing On Error
IF NOT DEFINED in_subprocess (CMD /k SET in_subprocess=y ^& %0 %*) & exit )

:: Expand Variables At Execution Time
SETLOCAL EnableDelayedExpansion

FOR /F "tokens=1,2 delims=#" %%a in ('"prompt #$H#$E# & echo on & for %%b in (1) do rem"') do (
  SET "DEL=%%a"
)
<NUL > X SET /p ".=."

:: Define Default Colors
COLOR 5E

:: Display Welcome Message
CALL :ECHO_WITH_COLOR 5E "Welcome to the WickedWhims Windows Installation"
CALL :ECHO_WITH_COLOR 5E "^!"
ECHO.
ECHO.

:: Define Documents Folder
FOR /F "tokens=3* delims= " %%a in ('reg query "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders" /v "Personal"') DO (SET MY_DOCUMENTS=%%a%%b)

:: Define EA Folder And Check If It Exist
SET EA_DIR=%MY_DOCUMENTS%"\Electronic Arts"

IF NOT EXIST %EA_DIR% (
    CALL :ECHO_WITH_COLOR CE "Please turn on The Sims 4 at least once before installing any mods"
    CALL :ECHO_WITH_COLOR CE "^!"
    ECHO.
    CALL :ECHO_WITH_COLOR CE "Electronic Arts folder is missing."
    ECHO.
    CALL :REMOVE_TEMP_X
    @PAUSE > NUL
    EXIT
)

:: Define Sims 4 Mods Folder
IF EXIST %EA_DIR%"\Die Sims 4" (
    SET SIMS_DIR=%EA_DIR%"\Die Sims 4"
) ELSE IF EXIST %EA_DIR%"\Los Sims 4" (
    SET SIMS_DIR=%EA_DIR%"\Los Sims 4"
) ELSE IF EXIST %EA_DIR%"\Les Sims 4" (
    SET SIMS_DIR=%EA_DIR%"\Les Sims 4"
) ELSE IF EXIST %EA_DIR%"\De Sims 4" (
    SET SIMS_DIR=%EA_DIR%"\De Sims 4"
) ELSE IF EXIST %EA_DIR%"\The Sims 4" (
    SET SIMS_DIR=%EA_DIR%"\The Sims 4"
) ELSE (
    CALL :ECHO_WITH_COLOR CE "Please turn on The Sims 4 at least once before installing any mods"
    CALL :ECHO_WITH_COLOR CE "^!"
    ECHO.
    CALL :ECHO_WITH_COLOR CE "The Sims 4 folder is missing."
    ECHO.
    CALL :REMOVE_TEMP_X
    @PAUSE > NUL
    EXIT
)

:: Remove Quote Marks From Path
SET SIMS_DIR=%SIMS_DIR:"=%

:: Test For Options File
IF NOT EXIST "%SIMS_DIR%\Options.ini" (
    CALL :ECHO_WITH_COLOR CE "Please turn on The Sims 4 at least once before installing any mods"
    CALL :ECHO_WITH_COLOR CE "^!"
    ECHO.
    CALL :ECHO_WITH_COLOR CE "Options.ini file is missing."
    ECHO.
    CALL :REMOVE_TEMP_X
    @PAUSE > NUL
    EXIT
)

:: Define Mods Folder Path
SET MODS_DIR=%SIMS_DIR%\Mods

:: Remove Quote Marks From Path
SET MODS_DIR=%MODS_DIR:"=%

:: Test If Mods Folder Exists
IF NOT EXIST "%MODS_DIR%" (
    CALL :ECHO_WITH_COLOR CE "Please turn on The Sims 4 at least once before installing any mods"
    CALL :ECHO_WITH_COLOR CE "^!"
    ECHO.
    CALL :ECHO_WITH_COLOR CE "The Sims 4 Mods folder is missing."
    ECHO.
    CALL :REMOVE_TEMP_X
    @PAUSE > NUL
    EXIT
)

:: Define Sims 4 folder and check if exist
ECHO Game Mods Folder: ^"%MODS_DIR%^"

:: Display current game version
:: WORKAROUND, NOT GUARANTEED TO WORK
IF EXIST "%SIMS_DIR%\Config.log" (
    :: Copy Config File To Root Dir As Temp
    XCOPY "%SIMS_DIR%\Config.log" . /Y > NUL

    :: Get First Line With 'Version'
    for /f "tokens=2 delims=:" %%v in (' type Config.log^|find /i "Version"') do (
        SET GAME_VERSION=%%v
        GOTO :END_LOOP
    )

    :END_LOOP
    ECHO > NUL

    :: Define Game Version Variable
    SET GAME_VERSION=%GAME_VERSION: =%
    ECHO Current Game Version: ^"%GAME_VERSION%^"

    :: Remove Temp Config File
    DEL Config.log

    :: Get The Second Number From The Version
    for /F "tokens=1,2 delims=. " %%a IN ("%GAME_VERSION%") DO SET /a VERSION_NUMBER=%%b

    :: Compare Version Numbers And Display Warning
    IF !VERSION_NUMBER! LSS %MIN_GAME_VERSION% (
        CALL :ECHO_WITH_COLOR CE "Warning! Your current game version is outdated"
        CALL :ECHO_WITH_COLOR CE "^!"
        ECHO.
        CALL :ECHO_WITH_COLOR CE "Update your game if you want to use WickedWhims."
        ECHO.
    )
)

:: Test For Mod Files Folder
IF NOT EXIST .\WickedWhimsMod (
    CALL :ECHO_WITH_COLOR CE "WickedWhims mod files are missing"
    CALL :ECHO_WITH_COLOR CE "^!"
    ECHO.
    CALL :ECHO_WITH_COLOR CE "Please extract all of the files from the archive."
    ECHO.
    CALL :REMOVE_TEMP_X
    @PAUSE > NUL
    EXIT
)

ECHO.
CALL :ECHO_WITH_COLOR 5B "Do you want to install WickedWhims?"
ECHO.
CALL :ECHO_WITH_COLOR 5E "Press any key to continue..."
ECHO.
@PAUSE > NUL
ECHO.

:: Enable Mods In Options
:: Define Options File Path
SET OPTIONS_FILE_PATH="%SIMS_DIR%\Options.ini"

:: Copy Options File To Root Dir As Temp
XCOPY %OPTIONS_FILE_PATH% . /Y > NUL

:: Create New Empty Options Files
COPY NUL Options.ini.temp2 > NUL

:: Iterate All Options File Lines And Replace Mods Flags
FOR /F "tokens=*" %%A IN (Options.ini) DO (
    IF "%%A" == "modsdisabled = 1" (
        >>Options.ini.temp2 ECHO modsdisabled = 0
    ) ELSE IF "%%A" == "scriptmodsenabled = 0" (
        >>Options.ini.temp2 ECHO scriptmodsenabled = 1
    ) ELSE (
        >>Options.ini.temp2 ECHO %%A
    )
)

:: Remove Temp Options File
DEL Options.ini

:: Rename And Copy New Options File
RENAME Options.ini.temp2 Options.ini
XCOPY Options.ini "%SIMS_DIR%" /Y > NUL
DEL Options.ini

CALL :ECHO_WITH_COLOR 5E "1. Enabled Custom Content and Script Mods in settings."
ECHO.

:: Create new Resource cfg file
DEL "%MODS_DIR%\Resource.cfg"

COPY NUL "%MODS_DIR%\Resource.cfg" > NUL

>>"%MODS_DIR%\Resource.cfg" ECHO Priority 500
>>"%MODS_DIR%\Resource.cfg" ECHO PackedFile *.package
>>"%MODS_DIR%\Resource.cfg" ECHO PackedFile */*.package
>>"%MODS_DIR%\Resource.cfg" ECHO PackedFile */*/*.package
>>"%MODS_DIR%\Resource.cfg" ECHO PackedFile */*/*/*.package
>>"%MODS_DIR%\Resource.cfg" ECHO PackedFile */*/*/*/*.package
>>"%MODS_DIR%\Resource.cfg" ECHO PackedFile */*/*/*/*/*.package

CALL :ECHO_WITH_COLOR 5E "2. Recreated the Resource.cfg file."
ECHO.

:: Prepare WickedWhims Mod Folder
SET WW_DIR="%MODS_DIR%\WickedWhimsMod"
IF NOT EXIST %WW_DIR% MKDIR %WW_DIR%

:: Go To Mod Files Folder
cd WickedWhimsMod

:: Copy Mod Files To The Mods Folder
XCOPY "TURBODRIVER_WickedWhims_Tuning.package" %WW_DIR% /Y > NUL
XCOPY "TURBODRIVER_WickedWhims_Scripts.ts4script" %WW_DIR% /Y > NUL
XCOPY "ww_cas_parts.json" %WW_DIR% /Y > NUL

DEL %WW_DIR%\TURBODRIVER_WickedWhims_Exhibitionism.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_Main.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_Nudity.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_Woohoo.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_CAS.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_CAS_Penis.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_CAS_Mannequin.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_Debug.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_Animations.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_Overrides.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_Overrides_Skin_Tones.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_Objects.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_WickedWhims_Relationships.package > NUL 2>&1
DEL %WW_DIR%\TURBODRIVER_Denton47_WickedWhims_RigBreastsOverride.package > NUL 2>&1

:: Go Back To The Root Folder
cd ..

CALL :ECHO_WITH_COLOR 5E "3. Copied all WickedWhims mod files."
ECHO.

ECHO.
CALL :ECHO_WITH_COLOR 5A "Done. Ready to Play"
CALL :ECHO_WITH_COLOR 5A "^!"
ECHO.
ECHO.

:: Remove The X File
CALL :REMOVE_TEMP_X

:: End
@PAUSE > NUL
EXIT

:: Remove Ref Temp File
:REMOVE_TEMP_X
DEL .\X > NUL

:: Echo Colored Text
:ECHO_WITH_COLOR
IF EXIST .\X (
    SET "param=^%~2" !
    SET "param=!param:"=\"!"
    FINDSTR /p /A:%1 "." "!param!\..\X" NUL
    <NUL SET /p ".=%DEL%%DEL%%DEL%%DEL%%DEL%%DEL%%DEL%"
    EXIT /b
)
